/*********************************************************************
 *  SPDX-License-Identifier: MIT                                     *
 *  Copyright (C) 2022-2023 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of qaplay.                                     *
 *********************************************************************/
#include "setplaylist.h"

SetPlayList::SetPlayList(QDialog* parent)
	: QDialog(parent)
{
	uiSpl.setupUi(&wSpl);
	uiSpl.pb_New->setIcon(QIcon("/usr/share/qaplay/icons/scalable/new.svg"));
	uiSpl.pb_FileDlg->setIcon(QIcon("/usr/share/qaplay/icons/scalable/open.svg"));
	uiSpl.pb_Rename->setIcon(QIcon("/usr/share/qaplay/icons/scalable/rename.svg"));
	uiSpl.pb_SaveAs->setIcon(QIcon("/usr/share/qaplay/icons/scalable/save-as.svg"));
	connect(uiSpl.pb_Set, SIGNAL(clicked()), this, SLOT(splDlgSet()));
	connect(uiSpl.pb_Cancel, SIGNAL(clicked()), this, SLOT(splDlgClose()));
	connect(uiSpl.pb_New, SIGNAL(clicked()), this, SLOT(splFileNew()));
	connect(uiSpl.pb_FileDlg, SIGNAL(clicked()), this, SLOT(splFileOpen()));
	connect(uiSpl.pb_Rename, SIGNAL(clicked()), this, SLOT(splFileRename()));
	connect(uiSpl.pb_SaveAs, SIGNAL(clicked()), this, SLOT(splFileSaveAs()));
	uiSpl.pb_Cancel->setFocus();
}

SetPlayList::~SetPlayList()
{
}

void SetPlayList::splDlgShow(const QString& sSplD)	// show song list, current dir or favourites songs
{
//cout << "SetPlayList - showSpl() reached " << endl;                                               // *****  debug  *****
	sSplFilePathNameCalled = sSplD;
	splDlgPopPathName(sSplFilePathNameCalled);

	wSpl.show();
}

void SetPlayList::splDlgSet()					// close dialog - set new m3u
{
	if (bSplCreateNew == true) {
		if (splFileNewCreate() == 0) {
			bSplCreateNew = false;
			splDlgPopPathName(sSplFilePathNameCalled);
			splFileCurChngd();
			return;
		}
	}
	emit sigSplQapM3uUpd(sSplFilePathName);
	emit sigSplQapDelete();
}

void SetPlayList::splDlgClose()
{
	emit sigSplQapDelete();
}

void SetPlayList::splDlgPopPathName(const QString& sSplD2)
{
	sSplFilePathName = sSplD2;
	QFileInfo fi(sSplFilePathName);
		sSplFilePath = fi.absolutePath();
		sSplFileName = fi.baseName();
	uiSpl.l_FilePath->setText(sSplFilePath);
	uiSpl.l_FileName->setText(sSplFileName);
}

void SetPlayList::splFileCurChngd()				// current m3u changed
{
	if (sSplFilePathName != sSplFilePathNameCalled) {
		uiSpl.pb_Set->setEnabled(true);
	} else {
		uiSpl.pb_Set->setEnabled(false);
	}
}

void SetPlayList::splFileNew()					// create new playlist to add files to
{
	bSplCreateNew = false;
	QFileDialog fd(this, tr("Create new, empty playlist"), sSplFilePath, "*.m3u");
		fd.setViewMode(QFileDialog::Detail);
		fd.setFileMode(QFileDialog::AnyFile);
		fd.setDefaultSuffix("m3u");
	
	if (fd.exec()) {
		slSplD = fd.selectedFiles();
	}
	if (slSplD.count() > 0) {						// dialog not canceled
		sSplFileFromDlg = slSplD.at(0);
		if (sSplFileFromDlg != "") {
			bSplCreateNew = true;
			splDlgPopPathName(sSplFileFromDlg);
			splFileCurChngd();
		} else {
			splQmb(1);								// empty/ no file name - should never happen
		}
	}
}

int SetPlayList::splFileNewCreate()				// physically create new playlist to add files to - 0=fail, 1=success
{
	if (m3uFileCreate(sSplFileFromDlg) == "0") {
		splQmb(11);									// new playlist could not be created
		return 0;
	} else {
		return 1;
	}
}

void SetPlayList::splFileOpen()					// open file dialog to pick an existing playlist to set
{
	sSplFileFromDlg = QFileDialog::getOpenFileName(this, tr("Select existing playlist"), sSplFilePath, "*.m3u");
	if (sSplFileFromDlg != "") {
		splDlgPopPathName(sSplFileFromDlg);
		splFileCurChngd();
	}
}

void SetPlayList::splFileRename()				// rename current playlist
{
	QFileDialog fd(this, tr("rename playlist"), sSplFilePath, "*.m3u");
		fd.setViewMode(QFileDialog::Detail);
		fd.setFileMode(QFileDialog::AnyFile);
		fd.selectUrl(QUrl::fromLocalFile(sSplFilePath + "/" + sSplFileName));
		fd.setDefaultSuffix("m3u");

	if (fd.exec() == 1) {
		slSplD = fd.selectedFiles();
	} else {
		return;
	}
	if (slSplD.count() > 0) {
		sSplFileFromDlg = slSplD.at(0);
	} else {
		return;
	}
//cout << "splFileRename - sSplFileRename = " << sSplFileFromDlg.toStdString() << endl;                                               // *****  debug  *****

	QFileInfo fi(sSplFileFromDlg);
	if (fi.exists() == true) {
		splQmb(21);									// invalid playlist name, already exists
	} else {
		QMessageBox msgBox;
//			msgBox.setWindowTitle( tr( "Rename ?" ) );
			msgBox.setIcon( QMessageBox::Question );
			msgBox.setText( tr( "\nSure to rename m3u ?\n" ) );
			msgBox.addButton( tr( "Yes, &Rename" ), QMessageBox::AcceptRole );
			msgBox.addButton( tr( "No, &Cancel" ), QMessageBox::RejectRole );
		if (msgBox.exec() == 0) {
			if (m3uFileRename(sSplFilePathName, sSplFileFromDlg) == false) {
				splQmb(22);								// error while renaming
			} else {
				splDlgPopPathName(sSplFileFromDlg);
				sSplFilePathName = sSplFileFromDlg;
				splQmb(23);								// playlist successfully renamed
				splDlgSet();
			}
		}
	}
}

void SetPlayList::splFileSaveAs()				// save current playlist under different name and start using it as such
{
	QFileDialog fd(this, tr("Save current playlist under new name"), sSplFilePath, "*.m3u");
		fd.setViewMode(QFileDialog::Detail);
		fd.setFileMode(QFileDialog::AnyFile);
		fd.setDefaultSuffix("m3u");
	
	if (fd.exec()) {
		slSplD = fd.selectedFiles();
	}
	sSplFileFromDlg = slSplD.at(0);

	if (sSplFileFromDlg != "") {
		if (m3uFileSaveAs(sSplFilePathName, sSplFileFromDlg) == true) {
			splDlgPopPathName(sSplFileFromDlg);
			sSplFilePathName = sSplFileFromDlg;
			splQmb(31);								// successfully saved under new name
			splDlgSet();
		} else {
			splQmb(32);								// error saving under new name
		}
	}
}

void SetPlayList::splQmb(int iSplD)				// QMessageBox
{
	QMessageBox msgBox;
		switch (iSplD) {
			case 1:									// splFileNew
				msgBox.setText("Error - Empty/no m3u playlist name - should never happen!?");
				msgBox.setInformativeText("Sigh, might want to try again?");
				break;
			case 11:								// splFileNewCreate
				msgBox.setText("Error - It seems, the new playlist could not be create.");
				msgBox.setInformativeText("May be the file exists or a write permission problem?");
				break;
			case 21:								// splFileNewRename - name invalid/already exists
				msgBox.setText("Error - Invalid playlist name.");
				msgBox.setInformativeText("A file with this name already exists!");
				break;
			case 22:								// splFileNewRename - error while renaming
				msgBox.setText("Error - while renaming playlist.");
				msgBox.setInformativeText("An error occurred while trying to rename the current playlist!");
				break;
			case 23:								// splFileNewRename - success
				msgBox.setText("Success - renaming playlist.");
				msgBox.setInformativeText("The playlist has been renamed!");
				break;
			case 31:								// splFileSaveAs - success
				msgBox.setText("Success - saving playlist under new name.");
				msgBox.setInformativeText("The playlist has been saved under the new name!");
				break;
			case 32:								// splFileSaveAs - error
				msgBox.setText("Error - It seems, the playlist could not be saved under the new name.");
				msgBox.setInformativeText("May be the file exists or a write permission problem?");
				break;
		}
		msgBox.exec();
}
