/*********************************************************************
 *  SPDX-License-Identifier: MIT                                     *
 *  Copyright (C) 2023 by Helge Kortendieck  alan@kasmithee.de       *
 *                                                                   *
 *  This file is part of qaplay.                                     *
 *  show internal song list functions                                *
 *********************************************************************/
#include "songlist.h"

SongList::SongList(QDialog* parent)
	: QDialog(parent)
{
	uiSl.setupUi(&wSl);
	uiSl.pb_Up->setIcon(QIcon("/usr/share/qaplay/icons/scalable/up.svg"));
	uiSl.pb_Down->setIcon(QIcon("/usr/share/qaplay/icons/scalable/down.svg"));
	uiSl.pb_Drop->setIcon(QIcon("/usr/share/qaplay/icons/scalable/drop.svg"));
	connect(uiSl.pb_Apply, SIGNAL(clicked()), this, SLOT(slDlgApply()));
	connect(uiSl.pb_Cancel, SIGNAL(clicked()), this, SLOT(slDlgCancel()));
	connect(uiSl.pb_Down, SIGNAL(clicked()), this, SLOT(slSongDown()));
	connect(uiSl.pb_Drop, SIGNAL(clicked()), this, SLOT(slSongDrop()));
	connect(uiSl.pb_Up, SIGNAL(clicked()), this, SLOT(slSongUp()));
}

SongList::~SongList()
{
}


void SongList::slDlgApply()
{
	emit sigSlQapSongChngd(uiSl.lw_SongList->currentRow());
	emit sigSlQapDelete();
}

void SongList::slDlgCancel()
{
	emit sigSlQapDelete();
}

void SongList::slDlgShow(int sliD, const QString& slsD, const QStringList& slslD)	// show song list dialog
{
//cout << "slDlgShow: reached " << endl;                                               // *****  debug  *****
	uiSl.l_SongList->setText(slsD);
	slslSongs = slslD;
	sliCount = slslSongs.count();
	sliCurIdx = sliD;
	slDlgPop();
	wSl.show();
}

void SongList::slDlgPop()						// populate list widget with songs
{
//cout << "slDlgPop: reached " << endl;                                               // *****  debug  *****
	uiSl.lw_SongList->clear();
	if (sliCount > 0) {
		for(sliD = 0; sliD < sliCount; sliD++) {
			QListWidgetItem *newSongItem = new QListWidgetItem(slslSongs.at(sliD));
			uiSl.lw_SongList->addItem(newSongItem);
		}
		uiSl.lw_SongList->setCurrentRow(sliCurIdx);
		connect(uiSl.lw_SongList, SIGNAL(currentRowChanged(int)), this, SLOT(slSongCurChngd(int)));
	}
}

void SongList::slSongCurChngd(int sliD)			// current song changed
{
	if (uiSl.lw_SongList->currentRow() != sliCurIdx) {
		uiSl.pb_Apply->setEnabled(true);
	} else {
		uiSl.pb_Apply->setEnabled(false);
	}
}

void SongList::slSongDown()						// move selected song down by 1 position
{
	sliD2 = uiSl.lw_SongList->currentRow();
//cout << "slSongDown: sliD2 = " << QString::number(sliD2).toStdString() << endl;                                               // *****  debug  *****
	if (sliD2 > -1 and sliD2 < sliCount -1) {
		slLiItems.move(sliD2, sliD2 + 1);
		sliD2 = sliD2 + 1;
		slDlgPop();
	}
}

void SongList::slSongDrop()						// drop selected song from song list
{
	sliD2 = uiSl.lw_SongList->currentRow();
	if (sliD2 > -1) {
		QMessageBox msgBox;
		msgBox.setText("Sure to drop the selected song from the song list?");
			msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
			msgBox.setDefaultButton(QMessageBox::No);
		sliRet = msgBox.exec();					// Yes=16384, No=65536 - sigh!
//cout << "slSongDrop: sliRet = " << QString::number(sliRet).toStdString() << endl;                                               // *****  debug  *****
		if (sliRet == 16384){
			slLiItems.removeAt(sliD2);
			sliCount = sliCount - 1;
			sliD2 = -1;
			slDlgPop();
		}
	}
}

void SongList::slSongUp()						// move selected song up by 1 position
{	
	sliD2 = uiSl.lw_SongList->currentRow();
	if (sliD2 > 0) {
		slLiItems.move(sliD2, sliD2 - 1);
		sliD2 = sliD2 - 1;
		slDlgPop();
	}
}
